@@ -32,6 +32,12 @@ html_context = {
         'https://quodlibet.github.io/fonts/font-mfizz.css',
         '_static/extra.css',
     ],
+    "display_gitlab": True,
+    "gitlab_user": "GNOME",
+    "gitlab_repo": "pygobject",
+    "gitlab_version": "master",
+    "conf_py_path": "/docs/",
+    "gitlab_host": "gitlab.gnome.org",
 }
 
 html_static_path = [
@@ -44,7 +50,11 @@ html_theme_options = {
 }
 
 extlinks = {
-    'gnomebug': ('https://bugzilla.gnome.org/show_bug.cgi?id=%s', '#'),
+    'bzbug': ('https://bugzilla.gnome.org/show_bug.cgi?id=%s', 'bz#'),
+    'issue': ('https://gitlab.gnome.org/GNOME/pygobject/issues/%s', 'issue#'),
+    'mr': (
+        'https://gitlab.gnome.org/GNOME/pygobject/merge_requests/%s', 'mr#'),
+    'user': ('https://gitlab.gnome.org/%s', ''),
 }
 
 suppress_warnings = ["image.nonlocal_uri"]
