@@ -60,11 +60,11 @@ static void
 _pyg_set_thread_block_funcs (PyGThreadBlockFunc block_threads_func,
 			     PyGThreadBlockFunc unblock_threads_func)
 {
-    PyGILState_STATE state = pyglib_gil_state_ensure ();
+    PyGILState_STATE state = PyGILState_Ensure ();
     PyErr_Warn (PyExc_DeprecationWarning,
                 "Using pyg_set_thread_block_funcs is not longer needed. "
                 "PyGObject always uses Py_BLOCK/UNBLOCK_THREADS.");
-    pyglib_gil_state_release (state);
+    PyGILState_Release (state);
 }
 
 /**
@@ -80,15 +80,15 @@ pyg_destroy_notify(gpointer user_data)
     PyObject *obj = (PyObject *)user_data;
     PyGILState_STATE state;
 
-    state = pyglib_gil_state_ensure();
+    state = PyGILState_Ensure();
     Py_DECREF(obj);
-    pyglib_gil_state_release(state);
+    PyGILState_Release(state);
 }
 
 
 /* ---------------- gobject module functions -------------------- */
 
-static PyObject *
+PyObject *
 pyg_type_name (PyObject *self, PyObject *args)
 {
     PyObject *gtype;
@@ -113,7 +113,7 @@ pyg_type_name (PyObject *self, PyObject *args)
     return NULL;
 }
 
-static PyObject *
+PyObject *
 pyg_type_from_name (PyObject *self, PyObject *args)
 {
     const gchar *name;
@@ -138,7 +138,7 @@ pyg_type_from_name (PyObject *self, PyObject *args)
     return NULL;
 }
 
-static PyObject *
+PyObject *
 pyg_type_is_a (PyObject *self, PyObject *args)
 {
     PyObject *gtype, *gparent;
@@ -166,12 +166,17 @@ pyg_object_set_property (GObject *object, guint property_id,
     PyObject *py_pspec, *py_value;
     PyGILState_STATE state;
 
-    state = pyglib_gil_state_ensure();
+    state = PyGILState_Ensure();
 
-    object_wrapper = pygobject_new(object);
+    object_wrapper = g_object_get_qdata(object, pygobject_wrapper_key);
+
+    if (object_wrapper)
+      Py_INCREF (object_wrapper);
+    else
+      object_wrapper = pygobject_new(object);
 
     if (object_wrapper == NULL) {
-	pyglib_gil_state_release(state);
+	PyGILState_Release(state);
 	return;
     }
 
@@ -190,7 +195,7 @@ pyg_object_set_property (GObject *object, guint property_id,
     Py_DECREF(py_pspec);
     Py_DECREF(py_value);
 
-    pyglib_gil_state_release(state);
+    PyGILState_Release(state);
 }
 
 static void
@@ -200,12 +205,17 @@ pyg_object_get_property (GObject *object, guint property_id,
     PyObject *object_wrapper, *retval;
     PyGILState_STATE state;
 
-    state = pyglib_gil_state_ensure();
+    state = PyGILState_Ensure();
 
-    object_wrapper = pygobject_new(object);
+    object_wrapper = g_object_get_qdata(object, pygobject_wrapper_key);
+
+    if (object_wrapper)
+      Py_INCREF (object_wrapper);
+    else
+      object_wrapper = pygobject_new(object);
 
     if (object_wrapper == NULL) {
-	pyglib_gil_state_release(state);
+	PyGILState_Release(state);
 	return;
     }
 
@@ -216,7 +226,7 @@ pyg_object_get_property (GObject *object, guint property_id,
     Py_DECREF(object_wrapper);
     Py_XDECREF(retval);
 
-    pyglib_gil_state_release(state);
+    PyGILState_Release(state);
 }
 
 typedef struct _PyGSignalAccumulatorData {
@@ -236,7 +246,7 @@ _pyg_signal_accumulator(GSignalInvocationHint *ihint,
     PyGSignalAccumulatorData *data = _data;
     PyGILState_STATE state;
 
-    state = pyglib_gil_state_ensure();
+    state = PyGILState_Ensure();
     if (ihint->detail)
         py_detail = PYGLIB_PyUnicode_FromString(g_quark_to_string(ihint->detail));
     else {
@@ -270,7 +280,7 @@ _pyg_signal_accumulator(GSignalInvocationHint *ihint,
         }
         Py_DECREF(py_retval);
     }
-    pyglib_gil_state_release(state);
+    PyGILState_Release(state);
     return retval;
 }
 
@@ -783,19 +793,19 @@ add_properties (GObjectClass *klass, PyObject *properties)
 	if (pspec) {
 	    g_object_class_install_property(klass, 1, pspec);
 	} else {
-            PyObject *type, *value, *traceback;
+            PyObject *type, *pvalue, *traceback;
 	    ret = FALSE;
-            PyErr_Fetch(&type, &value, &traceback);
-            if (PYGLIB_PyUnicode_Check(value)) {
+            PyErr_Fetch(&type, &pvalue, &traceback);
+            if (PYGLIB_PyUnicode_Check(pvalue)) {
                 char msg[256];
                 g_snprintf(msg, 256,
 			   "%s (while registering property '%s' for GType '%s')",
-               PYGLIB_PyUnicode_AsString(value),
+               PYGLIB_PyUnicode_AsString(pvalue),
 			   prop_name, G_OBJECT_CLASS_NAME(klass));
-                Py_DECREF(value);
+                Py_DECREF(pvalue);
                 value = PYGLIB_PyUnicode_FromString(msg);
             }
-            PyErr_Restore(type, value, traceback);
+            PyErr_Restore(type, pvalue, traceback);
 	    break;
 	}
     }
@@ -892,7 +902,7 @@ pyg_run_class_init(GType gtype, gpointer gclass, PyTypeObject *pyclass)
     return 0;
 }
 
-static PyObject *
+PyObject *
 _wrap_pyg_type_register(PyObject *self, PyObject *args)
 {
     PyTypeObject *class;
@@ -986,9 +996,10 @@ pygobject_constructv(PyGObject  *self,
 
     g_assert (self->obj == NULL);
     pygobject_init_wrapper_set((PyObject *) self);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     obj = g_object_newv(pyg_type_from_object((PyObject *) self),
                         n_parameters, parameters);
-
+G_GNUC_END_IGNORE_DEPRECATIONS
     if (g_object_is_floating (obj))
         self->private_flags.flags |= PYGOBJECT_GOBJECT_WAS_FLOATING;
     pygobject_sink (obj);
@@ -1021,7 +1032,7 @@ pygobject__g_instance_init(GTypeInstance   *instance,
            * g_object_new -> we have no python wrapper, so create it
            * now */
         PyGILState_STATE state;
-        state = pyglib_gil_state_ensure();
+        state = PyGILState_Ensure();
         wrapper = pygobject_new_full(object,
                                      /*steal=*/ FALSE,
                                      g_class);
@@ -1037,7 +1048,7 @@ pygobject__g_instance_init(GTypeInstance   *instance,
 
         Py_DECREF(args);
         Py_DECREF(kwargs);
-        pyglib_gil_state_release(state);
+        PyGILState_Release(state);
     }
 }
 
@@ -1214,7 +1225,7 @@ pyg_type_register(PyTypeObject *class, const char *type_name)
     return 0;
 }
 
-static PyObject *
+PyObject *
 pyg_signal_new(PyObject *self, PyObject *args)
 {
     gchar *signal_name;
@@ -1281,7 +1292,7 @@ pyg_signal_new(PyObject *self, PyObject *args)
     return NULL;
 }
 
-static PyObject *
+PyObject *
 pyg_object_class_list_properties (PyObject *self, PyObject *args)
 {
     GParamSpec **specs;
@@ -1338,7 +1349,7 @@ pyg_object_class_list_properties (PyObject *self, PyObject *args)
     return list;
 }
 
-static PyObject *
+PyObject *
 pyg_object_new (PyGObject *self, PyObject *args, PyObject *kwargs)
 {
     PyObject *pytype;
@@ -1370,7 +1381,10 @@ pyg_object_new (PyGObject *self, PyObject *args, PyObject *kwargs)
     if (!pygobject_prepare_construct_properties (class, kwargs, &n_params, &params))
         goto cleanup;
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     obj = g_object_newv(type, n_params, params);
+G_GNUC_END_IGNORE_DEPRECATIONS
+
     if (!obj)
 	PyErr_SetString (PyExc_RuntimeError, "could not create object");
 
@@ -1395,13 +1409,13 @@ pyg_object_new (PyGObject *self, PyObject *args, PyObject *kwargs)
 static int
 pygobject_gil_state_ensure (void)
 {
-    return pyglib_gil_state_ensure ();
+    return PyGILState_Ensure ();
 }
 
 static void
 pygobject_gil_state_release (int flag)
 {
-    pyglib_gil_state_release(flag);
+    PyGILState_Release(flag);
 }
 
 /* Only for backwards compatibility */
@@ -1411,7 +1425,7 @@ pygobject_enable_threads(void)
     return 0;
 }
 
-static PyObject *
+PyObject *
 pyg_signal_accumulator_true_handled(PyObject *unused, PyObject *args)
 {
     PyErr_SetString(PyExc_TypeError,
@@ -1433,7 +1447,7 @@ marshal_emission_hook(GSignalInvocationHint *ihint,
     PyObject *params;
     guint i;
 
-    state = pyglib_gil_state_ensure();
+    state = PyGILState_Ensure();
 
     /* construct Python tuple for the parameter values */
     params = PyTuple_New(n_param_values);
@@ -1464,11 +1478,11 @@ marshal_emission_hook(GSignalInvocationHint *ihint,
     retval = (retobj == Py_True ? TRUE : FALSE);
     Py_XDECREF(retobj);
 out:
-    pyglib_gil_state_release(state);
+    PyGILState_Release(state);
     return retval;
 }
 
-static PyObject *
+PyObject *
 pyg_add_emission_hook(PyGObject *self, PyObject *args)
 {
     PyObject *first, *callback, *extra_args, *data, *repr;
@@ -1526,7 +1540,7 @@ pyg_add_emission_hook(PyGObject *self, PyObject *args)
     return PyLong_FromUnsignedLong(hook_id);
 }
 
-static PyObject *
+PyObject *
 pyg__install_metaclass(PyObject *dummy, PyTypeObject *metaclass)
 {
     Py_INCREF(metaclass);
@@ -1539,7 +1553,7 @@ pyg__install_metaclass(PyObject *dummy, PyTypeObject *metaclass)
     return Py_None;
 }
 
-static PyObject *
+PyObject *
 pyg__gvalue_get(PyObject *module, PyObject *pygvalue)
 {
     if (!pyg_boxed_check (pygvalue, G_TYPE_VALUE)) {
@@ -1551,13 +1565,13 @@ pyg__gvalue_get(PyObject *module, PyObject *pygvalue)
                                   /*copy_boxed=*/ TRUE);
 }
 
-static PyObject *
+PyObject *
 pyg__gvalue_set(PyObject *module, PyObject *args)
 {
     PyObject *pygvalue;
     PyObject *pyobject;
 
-    if (!PyArg_ParseTuple (args, "OO:_gobject._gvalue_set",
+    if (!PyArg_ParseTuple (args, "OO:_gi._gvalue_set",
                            &pygvalue, &pyobject))
         return NULL;
 
@@ -1573,30 +1587,6 @@ pyg__gvalue_set(PyObject *module, PyObject *args)
     Py_RETURN_NONE;
 }
 
-static PyMethodDef _gobject_functions[] = {
-    { "type_name", pyg_type_name, METH_VARARGS },
-    { "type_from_name", pyg_type_from_name, METH_VARARGS },
-    { "type_is_a", pyg_type_is_a, METH_VARARGS },
-    { "type_register", _wrap_pyg_type_register, METH_VARARGS },
-    { "signal_new", pyg_signal_new, METH_VARARGS },
-    { "list_properties",
-      pyg_object_class_list_properties, METH_VARARGS },
-    { "new",
-      (PyCFunction)pyg_object_new, METH_VARARGS|METH_KEYWORDS },
-    { "signal_accumulator_true_handled",
-      (PyCFunction)pyg_signal_accumulator_true_handled, METH_VARARGS },
-    { "add_emission_hook",
-      (PyCFunction)pyg_add_emission_hook, METH_VARARGS },
-    { "_install_metaclass",
-      (PyCFunction)pyg__install_metaclass, METH_O },
-    { "_gvalue_get",
-      (PyCFunction)pyg__gvalue_get, METH_O },
-    { "_gvalue_set",
-      (PyCFunction)pyg__gvalue_set, METH_VARARGS },
-
-    { NULL, NULL, 0 }
-};
-
 
 /* ----------------- Constant extraction ------------------------ */
 
@@ -1616,7 +1606,7 @@ static PyMethodDef _gobject_functions[] = {
 const gchar *
 pyg_constant_strip_prefix(const gchar *name, const gchar *strip_prefix)
 {
-    gint prefix_len;
+    size_t prefix_len;
     guint i;
 
     prefix_len = strlen(strip_prefix);
@@ -1632,9 +1622,9 @@ pyg_constant_strip_prefix(const gchar *name, const gchar *strip_prefix)
 
     /* strip off prefix from value name, while keeping it a valid
      * identifier */
-    for (i = prefix_len; i >= 0; i--) {
-	if (g_ascii_isalpha(name[i]) || name[i] == '_') {
-	    return &name[i];
+    for (i = prefix_len + 1; i > 0; i--) {
+	if (g_ascii_isalpha(name[i - 1]) || name[i - 1] == '_') {
+	    return &name[i - 1];
 	}
     }
     return name;
@@ -1757,7 +1747,7 @@ pyg_parse_constructor_args(GType        obj_type,
         params[param_i].name = prop_names[arg_i];
         g_value_init(&params[param_i].value, spec->value_type);
         if (pyg_value_from_pyobject(&params[param_i].value, py_args[arg_i]) == -1) {
-            int i;
+            guint i;
             PyErr_Format(PyExc_TypeError, "could not convert parameter '%s' of type '%s'",
                          arg_names[arg_i], g_type_name(spec->value_type));
             g_type_class_unref(oclass);
@@ -1783,9 +1773,9 @@ _log_func(const gchar *log_domain,
 	PyGILState_STATE state;
 	PyObject* warning = user_data;
 
-	state = pyglib_gil_state_ensure();
+	state = PyGILState_Ensure();
 	PyErr_Warn(warning, (char *) message);
-	pyglib_gil_state_release(state);
+	PyGILState_Release(state);
     } else
         g_log_default_handler(log_domain, log_level, message, user_data);
 }
@@ -1893,12 +1883,7 @@ struct _PyGObject_Functions pygobject_api_functions = {
   pyg_flags_add,
   pyg_flags_from_gtype,
 
-  /* threads_enabled */
-#ifdef DISABLE_THREADING
-  FALSE,
-#else
-  TRUE,
-#endif
+  TRUE, /* threads_enabled */
 
   pygobject_enable_threads,
   pygobject_gil_state_ensure,
@@ -1925,7 +1910,7 @@ struct _PyGObject_Functions pygobject_api_functions = {
 };
 
 /* for addon libraries ... */
-static void
+void
 pygobject_register_api(PyObject *d)
 {
     PyObject *api;
@@ -1936,7 +1921,7 @@ pygobject_register_api(PyObject *d)
 }
 
 /* some constants */
-static void
+void
 pygobject_register_constants(PyObject *m)
 {
     /* PyFloat_ return a new ref, and add object takes the ref */
@@ -1968,7 +1953,7 @@ pygobject_register_constants(PyObject *m)
 }
 
 /* features */
-static void
+void
 pygobject_register_features(PyObject *d)
 {
     PyObject *features;
@@ -1979,7 +1964,7 @@ pygobject_register_features(PyObject *d)
     Py_DECREF(features);
 }
 
-static void
+void
 pygobject_register_version_tuples(PyObject *d)
 {
     PyObject *tuple;
@@ -1992,7 +1977,7 @@ pygobject_register_version_tuples(PyObject *d)
     PyDict_SetItemString(d, "pygobject_version", tuple);
 }
 
-static void
+void
 pygobject_register_warnings(PyObject *d)
 {
     PyObject *warning;
@@ -2003,25 +1988,3 @@ pygobject_register_warnings(PyObject *d)
     add_warning_redirection("GLib-GObject", warning);
     add_warning_redirection("GThread", warning);
 }
-
-
-PYGLIB_MODULE_START(_gobject, "_gobject")
-{
-    PyObject *d;
-
-    d = PyModule_GetDict(module);
-    pygobject_register_api(d);
-    pygobject_register_constants(module);
-    pygobject_register_features(d);
-    pygobject_register_version_tuples(d);
-    pygobject_register_warnings(d);
-    pygobject_type_register_types(d);
-    pygobject_object_register_types(d);
-    pygobject_interface_register_types(d);
-    pygobject_paramspec_register_types(d);
-    pygobject_boxed_register_types(d);
-    pygobject_pointer_register_types(d);
-    pygobject_enum_register_types(d);
-    pygobject_flags_register_types(d);
-}
-PYGLIB_MODULE_END
